#!/usr/bin/env python
# License: PSF
# see: LICENSE
# for full text of the license
#
major_version = 0
mid_version = 0
minor_version = 6
version='%s.%s.%s' % (major_version, mid_version, minor_version)
